<?php

/**
 * Class GirdGallery_Galleries_Module
 *
 * @package GirdGallery\Galleries
 * @author Artur Kovalevsky
 */
class GirdGallery_Galleries_Module extends Rsc_Mvc_Module
{

    /**
     * {@inheritdoc}
     */
    public function onInit()
    {
        parent::onInit();

        $resources = new GirdGallery_Galleries_Model_Resources();

        $config = $this->getEnvironment()->getConfig();
        $prefix = $config->get('hooks_prefix');

        /* Loads assets when UI loaded */
        add_action($prefix . 'after_ui_loaded', array($this, 'loadAssets'));

        /* Deletes gallery */
        add_action($prefix . 'gallery_delete', array(
            $resources, 'deleteByGalleryId'
        ));

        /* Delete attachment */
        add_action(
            'gird_gallery_delete_image',
            array(
                $resources,
                'deleteByResourceId'
            )
        );

        add_shortcode('gird-gallery', array($this, 'getGallery'));

        add_image_size('gg_gallery_thumbnail', 350, 150, true);

        $lang    = $this->getEnvironment()->getLang();
        $menu    = $this->getEnvironment()->getMenu();
        $submenu = $menu->createSubmenuItem();

        $submenu->setCapability('manage_options')
            ->setMenuSlug('gird-gallery-new')
            ->setMenuTitle($lang->translate('New gallery'))
            ->setPageTitle($lang->translate('New gallery'))
            ->setModuleName('galleries');

        $menu->addSubmenuItem('newGallery', $submenu)
            ->register();
    }

    /**
     * Loads assets
     */
    public function loadAssets(GirdGallery_Ui_Module $ui)
    {
        /* CSS */
        $ui->add(new GirdGallery_Ui_BackendStylesheet(
            'gg-galleries-style',
            $this->getLocationUrl() . '/assets/css/gird-gallery.galleries.style.css')
        );

        $ui->add(new GirdGallery_Ui_Stylesheet(
            'gg-galleries-frontend',
            $this->getLocationUrl() . '/assets/css/gird-gallery.galleries.frontend.css'
        ));

        $ui->add(new GirdGallery_Ui_Stylesheet(
            'gg-galleries-effects',
            $this->getLocationUrl() . '/assets/css/gird-gallery.galleries.effects.css'
        ));

        /* JS */

        $ui->add(new GirdGallery_Ui_BackendJavascript(
            'gg-galleries-index-js',
            $this->getLocationUrl() . '/assets/js/gird-gallery.galleries.index.js'
        ));

        $ui->add(new GirdGallery_Ui_BackendJavascript(
            'gg-galleries-view-js',
            $this->getLocationUrl() . '/assets/js/gird-gallery.galleries.view.js'
        ));

        $ui->add(new GirdGallery_Ui_BackendJavascript(
            'gg-galleries-preview-js',
            $this->getLocationUrl() . '/assets/js/gird-gallery.galleries.preview.js'
        ));

        $ui->add(new GirdGallery_Ui_BackendJavascript(
            'gg-galleries-thumb-js',
            $this->getLocationUrl() . '/assets/js/gird-gallery.galleries.thumb.js'
        ));

        $ui->add(new GirdGallery_Ui_BackendJavascript(
            'gg-gallery-settings',
            $this->getLocationUrl() . '/assets/js/settings.js'
        ));

        $ui->add(
            new GirdGallery_Ui_BackendJavascript(
                'gg-gallery-images',
                $this->getLocationUrl() . '/assets/js/addImages.js'
            )
        );

        /* Frontend javascripts */
        $frontend = new GirdGallery_Ui_Javascript(
            'gg-galleries-frontend-js',
            $this->getLocationUrl() . '/assets/js/gird-gallery.galleries.frontend.js'
        );

        $frontend->setDeps(array('jquery'));

        $ui->add(new GirdGallery_Ui_Javascript('jquery'));
        $ui->add($frontend);
    }

    /**
     * Shortcode callback.
     *
     * @param array $attributes An array of the shortcode parameters.
     *
     * @return string
     */
    public function getGallery($attributes)
    {
        $galleries = new GirdGallery_Galleries_Model_Galleries();
        $twig      = $this->getEnvironment()->getTwig();
        $cache     = $this->getEnvironment()->getCache();
        $gallery   = $galleries->getById($attributes['id']);

        $attachment = new GirdGallery_Galleries_Attachment();
        $twig->addFunction(
            new Twig_SimpleFunction('get_attachment', array(
                $attachment,
                'getAttachment'
            ))
        );

        $key = sprintf('gallery_settings_%s', $attributes['id']);

        /** @var GirdGallery_Settings_Registry $registry */
        $registry = $this->getEnvironment()
            ->getModule('settings')
            ->getRegistry();

        if (true === (bool) $registry->get('cache_enabled')) {
            $ttl = $registry->get('cache_ttl');
            $cache->setTtl($ttl);

            if (null === $settings = $cache->get($key)) {
                $settings = $this->getGallerySettings($attributes['id']);
                $cache->set($key, $settings, (int) $ttl);
            }

        } else {
            $settings = $this->getGallerySettings($attributes['id']);
        }

        $colorbox = $this->getEnvironment()->getModule('colorbox');

        $template = $twig->render(
            '@galleries/shortcode/gallery.twig',
            array(
                'gallery'  => $gallery,
                'settings' => $settings,
                'colorbox' => $colorbox->getLocationUrl(),
            )
        );

        return $template;
    }

    public function getGallerySettings($galleryId)
    {
        $model = new GirdGallery_Galleries_Model_Settings();

        if (null === $settings = $model->get((int) $galleryId)->data) {
            $config = $this->getEnvironment()->getConfig();
            $config->load('@galleries/settings.php');

            $settings = unserialize($config->get('gallery_settings'));
        }

        return $settings;
    }
}
